# Generated by Django 4.2.13 on 2024-05-29 16:05

from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    dependencies = [
        ('eveonline', '0017_alliance_and_corp_names_are_not_unique'),
        ('aa_contacts', '0001_initial'),
    ]

    operations = [
        migrations.AlterModelOptions(
            name='general',
            options={'default_permissions': (), 'managed': False, 'permissions': (('manage_alliance_contacts', 'Can manage alliance contacts'), ('manage_corporation_contacts', 'Can manage corporation contacts'), ('view_alliance_notes', 'Can view notes on alliance contacts'), ('view_corporation_notes', 'Can view notes on corporation contacts'))},
        ),
        migrations.AlterField(
            model_name='alliancecontact',
            name='labels',
            field=models.ManyToManyField(blank=True, related_name='contacts', to='aa_contacts.alliancecontactlabel'),
        ),
        migrations.CreateModel(
            name='CorporationContactLabel',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('label_id', models.BigIntegerField()),
                ('label_name', models.CharField(max_length=255)),
                ('corporation', models.ForeignKey(on_delete=django.db.models.deletion.RESTRICT, related_name='contact_labels', to='eveonline.evecorporationinfo')),
            ],
            options={
                'default_permissions': (),
            },
        ),
        migrations.CreateModel(
            name='CorporationContact',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('contact_id', models.BigIntegerField()),
                ('contact_type', models.CharField(choices=[('character', 'Character'), ('corporation', 'Corporation'), ('alliance', 'Alliance'), ('faction', 'Faction')], max_length=11)),
                ('standing', models.FloatField()),
                ('notes', models.TextField(blank=True, default='')),
                ('corporation', models.ForeignKey(on_delete=django.db.models.deletion.RESTRICT, related_name='contacts', to='eveonline.evecorporationinfo')),
                ('labels', models.ManyToManyField(blank=True, related_name='contacts', to='aa_contacts.corporationcontactlabel')),
            ],
            options={
                'default_permissions': (),
            },
        ),
    ]
